@@ -79,7 +79,7 @@ int _yr_ac_queue_push(
   pushed_node = (QUEUE_NODE*) yr_malloc(sizeof(QUEUE_NODE));
 
   if (pushed_node == NULL)
-    return ERROR_INSUFICIENT_MEMORY;
+    return ERROR_INSUFFICIENT_MEMORY;
 
   pushed_node->previous = queue->tail;
   pushed_node->next = NULL;
@@ -480,11 +480,11 @@ int _yr_ac_find_suitable_transition_table_slot(
       automaton->t_table = (YR_AC_TRANSITION_TABLE) yr_realloc(
           automaton->t_table, t_bytes_size * 2);
 
-	  automaton->m_table = (YR_AC_MATCH_TABLE) yr_realloc(
+      automaton->m_table = (YR_AC_MATCH_TABLE) yr_realloc(
           automaton->m_table, m_bytes_size * 2);
 
       if (automaton->t_table == NULL || automaton->m_table == NULL)
-        return ERROR_INSUFICIENT_MEMORY;
+        return ERROR_INSUFFICIENT_MEMORY;
 
       memset((uint8_t*) automaton->t_table + t_bytes_size, 0, t_bytes_size);
       memset((uint8_t*) automaton->m_table + m_bytes_size, 0, m_bytes_size);
@@ -548,7 +548,7 @@ int _yr_ac_find_suitable_transition_table_slot(
 // link must be used instead.
 //
 // The transition table for state S starts at T[S] and spans the next 257
-// slots in the array (1 for the failure link and 256 for all the posible
+// slots in the array (1 for the failure link and 256 for all the possible
 // transitions). But many of those slots are for invalid transitions, so
 // the transitions for multiple states can be interleaved as long as they don't
 // collide. For example, instead of having this transition table with state S1
@@ -605,7 +605,7 @@ int _yr_ac_build_transition_table(
     yr_free(automaton->t_table);
     yr_free(automaton->m_table);
 
-    return ERROR_INSUFICIENT_MEMORY;
+    return ERROR_INSUFFICIENT_MEMORY;
   }
 
   memset(automaton->t_table, 0,
@@ -775,7 +775,7 @@ int yr_ac_automaton_create(
     yr_free(new_automaton);
     yr_free(root_state);
 
-    return ERROR_INSUFICIENT_MEMORY;
+    return ERROR_INSUFFICIENT_MEMORY;
   }
 
   root_state->depth = 0;
@@ -850,7 +850,7 @@ int yr_ac_add_string(
         next_state = _yr_ac_state_create(state, atom->atom[i]);
 
         if (next_state == NULL)
-          return ERROR_INSUFICIENT_MEMORY;
+          return ERROR_INSUFFICIENT_MEMORY;
       }
 
       state = next_state;
