@@ -58,7 +58,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 typedef void* yyscan_t;
 #endif
 
-#define YY_EXTRA_TYPE RE*
+#define YY_EXTRA_TYPE RE_AST*
 #define YY_USE_CONST
 
 
@@ -105,6 +105,5 @@ void yyfatal(
 
 int yr_parse_re_string(
   const char* re_string,
-  int flags,
-  RE** re,
+  RE_AST** re_ast,
   RE_ERROR* error);
