@@ -120,7 +120,9 @@ end_declarations;
 int module_initialize(
     YR_MODULE* module)
 {
-  for (int i = 0; i < MAX_THREADS; i++)
+  int i;
+
+  for (i = 0; i < MAX_THREADS; i++)
     magic_cookie[i] = NULL;
 
   return ERROR_SUCCESS;
@@ -130,7 +132,9 @@ int module_initialize(
 int module_finalize(
     YR_MODULE* module)
 {
-  for (int i = 0; i < MAX_THREADS; i++)
+  int i;
+
+  for (i = 0; i < MAX_THREADS; i++)
     if (magic_cookie[i] != NULL)
       magic_close(magic_cookie[i]);
 
@@ -161,7 +165,7 @@ int module_load(
     }
     else
     {
-      return ERROR_INSUFICIENT_MEMORY;
+      return ERROR_INSUFFICIENT_MEMORY;
     }
   }
 
