@@ -38,8 +38,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <windows.h>
 
-#define PRIx64 "llx"
-#define PRId64 "lld"
+#define PRIx64 "I64x"
+#define PRId64 "I64d"
 
 #endif
 
@@ -53,7 +53,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "args.h"
 #include "threading.h"
-#include "config.h"
 
 
 #define ERROR_COULD_NOT_CREATE_THREAD  100
@@ -82,6 +81,7 @@ typedef struct _MODULE_DATA
 
 } MODULE_DATA;
 
+
 typedef struct _THREAD_ARGS
 {
   YR_RULES* rules;
@@ -90,41 +90,49 @@ typedef struct _THREAD_ARGS
 } THREAD_ARGS;
 
 
-typedef struct _QUEUED_FILE {
-
+typedef struct _QUEUED_FILE
+{
   char* path;
 
 } QUEUED_FILE;
 
 
+typedef struct COMPILER_RESULTS
+{
+  int errors;
+  int warnings;
+
+} COMPILER_RESULTS;
+
+
 #define MAX_ARGS_TAG            32
 #define MAX_ARGS_IDENTIFIER     32
 #define MAX_ARGS_EXT_VAR        32
 #define MAX_ARGS_MODULE_DATA    32
 
-char* tags[MAX_ARGS_TAG + 1];
-char* identifiers[MAX_ARGS_IDENTIFIER + 1];
-char* ext_vars[MAX_ARGS_EXT_VAR + 1];
-char* modules_data[MAX_ARGS_EXT_VAR + 1];
+static char* tags[MAX_ARGS_TAG + 1];
+static char* identifiers[MAX_ARGS_IDENTIFIER + 1];
+static char* ext_vars[MAX_ARGS_EXT_VAR + 1];
+static char* modules_data[MAX_ARGS_EXT_VAR + 1];
 
-int recursive_search = FALSE;
-int show_module_data = FALSE;
-int show_tags = FALSE;
-int show_specified_tags = FALSE;
-int show_specified_rules = FALSE;
-int show_strings = FALSE;
-int show_meta = FALSE;
-int show_namespace = FALSE;
-int show_version = FALSE;
-int show_help = FALSE;
-int ignore_warnings = FALSE;
-int fast_scan = FALSE;
-int negate = FALSE;
-int count = 0;
-int limit = 0;
-int timeout = 1000000;
-int stack_size = DEFAULT_STACK_SIZE;
-int threads = 8;
+static int recursive_search = FALSE;
+static int show_module_data = FALSE;
+static int show_tags = FALSE;
+static int show_strings = FALSE;
+static int show_string_length = FALSE;
+static int show_meta = FALSE;
+static int show_namespace = FALSE;
+static int show_version = FALSE;
+static int show_help = FALSE;
+static int ignore_warnings = FALSE;
+static int fast_scan = FALSE;
+static int negate = FALSE;
+static int count = 0;
+static int limit = 0;
+static int timeout = 1000000;
+static int stack_size = DEFAULT_STACK_SIZE;
+static int threads = MAX_THREADS;
+static int fail_on_warnings = FALSE;
 
 
 #define USAGE_STRING \
@@ -154,6 +162,9 @@ args_option_t options[] =
   OPT_BOOLEAN('s', "print-strings", &show_strings,
       "print matching strings"),
 
+  OPT_BOOLEAN('L', "print-string-length", &show_string_length,
+      "print length of matched strings"),
+
   OPT_BOOLEAN('e', "print-namespace", &show_namespace,
       "print rules' namespace"),
 
@@ -184,6 +195,9 @@ args_option_t options[] =
   OPT_BOOLEAN('w', "no-warnings", &ignore_warnings,
       "disable warnings"),
 
+  OPT_BOOLEAN(0, "fail-on-warnings", &fail_on_warnings,
+      "fail on warnings"),
+
   OPT_BOOLEAN('v', "version", &show_version,
       "show version information"),
 
@@ -467,12 +481,9 @@ void print_hex_string(
     int length)
 {
   for (int i = 0; i < min(32, length); i++)
-    printf("%02X ", (uint8_t) data[i]);
+    printf("%s%02X", (i == 0 ? "" : " "), (uint8_t) data[i]);
 
-  if (length > 32)
-    printf("...");
-
-  printf("\n");
+  puts(length > 32 ? " ..." : "");
 }
 
 
@@ -486,7 +497,7 @@ void print_scanner_error(
     case ERROR_COULD_NOT_ATTACH_TO_PROCESS:
       fprintf(stderr, "can not attach to process (try running as root)\n");
       break;
-    case ERROR_INSUFICIENT_MEMORY:
+    case ERROR_INSUFFICIENT_MEMORY:
       fprintf(stderr, "not enough memory\n");
       break;
     case ERROR_SCAN_TIMEOUT:
@@ -526,10 +537,12 @@ void print_compiler_error(
   {
     fprintf(stderr, "%s(%d): error: %s\n", file_name, line_number, message);
   }
-  else
+  else if (!ignore_warnings)
   {
-    if (!ignore_warnings)
-      fprintf(stderr, "%s(%d): warning: %s\n", file_name, line_number, message);
+    COMPILER_RESULTS* compiler_results = (COMPILER_RESULTS*) user_data;
+    compiler_results->warnings++;
+
+    fprintf(stderr, "%s(%d): warning: %s\n", file_name, line_number, message);
   }
 }
 
@@ -644,7 +657,7 @@ int handle_message(
 
     // Show matched strings.
 
-    if (show_strings)
+    if (show_strings || show_string_length)
     {
       YR_STRING* string;
 
@@ -654,14 +667,29 @@ int handle_message(
 
         yr_string_matches_foreach(string, match)
         {
-          printf("0x%" PRIx64 ":%s: ",
+          if (show_string_length)
+            printf("0x%" PRIx64 ":%d:%s",
+              match->base + match->offset,
+              match->data_length,
+              string->identifier);
+          else
+            printf("0x%" PRIx64 ":%s",
               match->base + match->offset,
               string->identifier);
 
-          if (STRING_IS_HEX(string))
-            print_hex_string(match->data, match->data_length);
+          if (show_strings)
+          {
+            printf(": ");
+
+            if (STRING_IS_HEX(string))
+              print_hex_string(match->data, match->data_length);
+            else
+              print_string(match->data, match->data_length);
+          }
           else
-            print_string(match->data, match->data_length);
+          {
+            printf("\n");
+          }
         }
       }
     }
@@ -984,16 +1012,18 @@ int main(
     int argc,
     const char** argv)
 {
+  COMPILER_RESULTS cr;
+
   YR_COMPILER* compiler = NULL;
   YR_RULES* rules = NULL;
 
-  int result;
+  int result, i;
 
   argc = args_parse(options, argc, argv);
 
   if (show_version)
   {
-    printf("%s\n", PACKAGE_STRING);
+    printf("%s\n", YR_VERSION);
     return EXIT_SUCCESS;
   }
 
@@ -1003,14 +1033,20 @@ int main(
       "YARA %s, the pattern matching swiss army knife.\n"
       "%s\n\n"
       "Mandatory arguments to long options are mandatory for "
-      "short options too.\n\n", PACKAGE_VERSION, USAGE_STRING);
+      "short options too.\n\n", YR_VERSION, USAGE_STRING);
 
     args_print_usage(options, 35);
-    printf("\nSend bug reports and suggestions to: %s.\n", PACKAGE_BUGREPORT);
+    printf("\nSend bug reports and suggestions to: vmalvarez@virustotal.com.\n");
 
     return EXIT_SUCCESS;
   }
 
+  if (threads > MAX_THREADS)
+  {
+    fprintf(stderr, "maximum number of threads is %d\n", MAX_THREADS);
+    return EXIT_FAILURE;
+  }
+
   if (argc != 2)
   {
     // After parsing the command-line options we expect two additional
@@ -1085,7 +1121,10 @@ int main(
       exit_with_code(EXIT_FAILURE);
     }
 
-    yr_compiler_set_callback(compiler, print_compiler_error, NULL);
+    cr.errors = 0;
+    cr.warnings = 0;
+
+    yr_compiler_set_callback(compiler, print_compiler_error, &cr);
 
     FILE* rule_file = fopen(argv[0], "r");
 
@@ -1095,11 +1134,14 @@ int main(
       exit_with_code(EXIT_FAILURE);
     }
 
-    int errors = yr_compiler_add_file(compiler, rule_file, NULL, argv[0]);
+    cr.errors = yr_compiler_add_file(compiler, rule_file, NULL, argv[0]);
 
     fclose(rule_file);
 
-    if (errors > 0)
+    if (cr.errors > 0)
+      exit_with_code(EXIT_FAILURE);
+
+    if (fail_on_warnings && cr.warnings > 0)
       exit_with_code(EXIT_FAILURE);
 
     result = yr_compiler_get_rules(compiler, &rules);
@@ -1152,7 +1194,7 @@ int main(
     thread_args.rules = rules;
     thread_args.start_time = start_time;
 
-    for (int i = 0; i < threads; i++)
+    for (i = 0; i < threads; i++)
     {
       if (create_thread(&thread[i], scanning_thread, (void*) &thread_args))
       {
@@ -1171,7 +1213,7 @@ int main(
     file_queue_finish();
 
     // Wait for scan threads to finish
-    for (int i = 0; i < threads; i++)
+    for (i = 0; i < threads; i++)
       thread_join(&thread[i]);
 
     file_queue_destroy();
